' MEMLOAD2.BAS                  (c) 1997 by Charles J. Olson
'
' by: Chuck Olson
'     WB9KZY
'     Jackson Harbor Press
'     RR1, Box 91C
'     Washington Island, WI  54246
'     email:  jacksonharbor@worldnet.att.net
'
' Note:  MEMLOAD.BAS ==> MEMLOAD2.BAS -  An 8 dah character (colon) needed 
'        for embedding a pause with the Island Memory kit was added. 
'        4/15/97 CJO
'
'  ***********
'  * WARNING *
'  ***********
'
'  This program has only been tried by me on three PC clones - a
'  Headstart XT, an Acer 486 SX and a Tenex 486SLC-2.  It has worked
'  on these machines without incident.  However, it may not work on
'  your computer/keyer or it may even kill your machine and/or keyer,
'  dead!
'
'  Proceed at your own risk!
'
'
'   Introduction:
'
'   One of first things I noticed when I first started using my memory
'   keyer was that it was nearly impossible for me to enter a long
'   message (>50 characters) without making some kind of mistake.
'   This is probably due to a combination of my lack of expertise with
'   CW and the unforgiving nature of my keyer design.
'   Unfortunately, my keyer design doesn't have any editing capability
'   (I ran out of room in the microcontroller EPROM) so my only choice
'   was to re-enter my entire message, this time without a mistake.
'    After struggling with this for a while I remembered the often
'   repeated words of one of my former bosses: "Can't we automate this,
'   somehow?".  This QBASIC program is my attempt to "automate" the
'   loading or recording of a keyer memory.  While the program was
'   developed for my WB9KZY memory keyer, it should work with any
'   memory keyer that uses a 5 volt power supply and has a sidetone
'   output.
'
'   Interface:
'
'   This program works by using two open-collector outputs from a PC
'   LPT printer port to actuate the dit and dah paddle inputs of the
'   keyer.  An input pin on the LPT port is connected to a tone detector
'   circuit which is in turn connected to the sidetone output on the
'   keyer chip.
'   You may not need the 74HC04 buffer chip, it depends on how "strong"
'   the pullups are on your LPT port.  You may also use a 74HC14 in place
'   of the 74HC04.  The HC14 has Schmidt trigger (hysteresis) input
'   circuitry. The hardware required is shown in the following diagram:
'
'      lpt pin 14  >---------------------------------->  keyer dit input
'       (/autofeed)
'
'      lpt pin 16  >---------------------------------->  keyer dah input
'       (/init)
'
'         74HC04        1N34                   74HC04
'pins:   13  12 11  10   | \  |        pins: 1   2  3   4
'          |\     |\     |  \ |               |\     |\
'keyer >---| >o---| >o---|   >|---x-------x---| >o---| >o--> lpt pin 11
'chip      |/     |/     |  / |   |       |   |/     |/       (Busy)
'sidetone                | /  |   |       |
'output pin                       |      <
'(pin 3 of WB9KZY keyer chip)     |       \
'                             ---------    >
'                       .1 uf ---------   /   10 k ohms
'                                 |      <
'                                 |       \
'                                 |        >
'                                 |       /
'                                 |      <
'                                 |       |
'                                 |       |
'   lpt pins 18-25 >--------------+-------+-------------> keyer ground
'        (ground)
'
'
'       unused gates on 74HC04             power connection on 74HC04
'pins:  9    8             5   6            pin 7: ground  pin 14: +5V
'        |\                 |\            
' gnd>---| >o--->    gnd>---| >o--->        
'        |/                 |/             
'
'
'
'      LPT 25 pin D connector(rear view):
'
'        13  12  11  10   9   8   7   6   5   4   3   2   1
'      -------------------------------------------------------
'       \ o   o   o   o   o   o   o   o   o   o   o   o   o /
'        \  o   o   o   o   o   o   o   o   o   o   o   o /
'         ------------------------------------------------
'          25  24  23  22  21  20  19  18  17  16  15  14
'
' The sidetone from the keyer is needed so that a minimum of timing
' has to be done by the program.  The program keys the keyer as you
' would, by pressing the paddle and "listening" to the sidetone
' to determine when to leave go of the paddle. 
'
' But I did have to do a little timing to get the correct spacing
' between characters and words.  The number displayed on the right
' side of the screen as you play a message is the dit count.  You
' might have to set the initial dit count for your computer and
' keyer speed combination - this is needed only for correctly
' spacing a message with a "T" or "E" at the beginning - otherwise
' the T or E would run into the next character.
'
' I detect the sidetone because with my keyer (and others) the
' transmitter output is disabled during the memory programming
' process - otherwise I would use the xmtr output. 

'
'   Operation:
'
'   Invoke the DOS program, MSD, to find out the port address of your
'   LPT port.  This should be either 03BCH, 0378H or 0278H.
'
'   Use a text editor, like the DOS program, EDIT, to create a file
'   containing your desired message.  A word space will be "sent"
'   when any spaces are encountered in the file.  Other
'   special characters allowed in the text include:
'       ?
'       , (comma)
'       - (bt)
'       . (period)
'       / (-..-.)
'       = (ar)
'       ; (sk)
'       \ (as)
'       : (8 dahs for embedding a pause with the Island Memory)
'  All other characters should be ignored by the program.
'
'
'   To invoke the program the first time type:  QBASIC MEMLOAD
'
'   Uncomment the appropriate line below which corresponds to the
'   base address of your LPT port by deleting the ' in the first column.
CLS
'baseadr = 888   'used for Port Address 0378H
'baseadr = 956   'used for Port Address 03BCH
'baseadr = 632   'used for Port Address 0278H
'
'   Then save the program.
'
'   Connect the interface hardware to the PC and the keyer - then
'   run the program to check things out.
'
'   Hit the esc key to bail out of the memory play routine.  If esc
'   doesn't work, you will have to reset the PC.
'
'   To program the keyer memory in question you will have to either press
'   the appropriate keys (on the keyer) or embed the appropriate codes
'   (in the message text file) or both prior
'   to entering the message text file name.
'
IF baseadr = 0 THEN PRINT "You need to select the LPT Base Address!!" ELSE 0
STOP  'see the verbage above if the program stops here.
0 statin = baseadr + 1
ctrlout = baseadr + 2
DIM a(122, 2) ' morse code table
GOSUB 60
ON ERROR GOTO 55
1 DIM aa$(38)
2 PRINT "enter input file name"
3 INPUT INFILE$
4  OPEN "I", 1, INFILE$
REM dit = 256  'for 486
dit = 30       'for XT
count = 0
5 x$ = INPUT$(1, #1)
6 p = ASC(x$)

7 count = count + 1
PRINT count, x$, p, dit
FOR I = 1 TO a(p, 2)
bit = a(p, 1) AND 2 ^ (8 - I)
IF bit > 0 THEN GOSUB 15 ELSE GOSUB 35
NEXT I
OUT ctrlout, 4 'turns off both the paddles
del = 3 * dit
8  stat = INP(statin) AND 128
   del = del - 1
   IF del > 0 THEN 8
IF INKEY$ = CHR$(27) THEN END   '27 is the ASCII code for Esc.
GOTO 5
END

REM pin 16, dah, (bit 2) is pos logic and must be set to 1 to output a 1
REM pin 14, dit, (bit 1) is neg logic and must be set to 0 to output a 1

REM     dah routine
15    OUT ctrlout, 0 'turn on one paddle
20    stat = INP(statin) AND 128
    IF stat = 128 THEN 20
30  stat = INP(statin) AND 128
    IF stat = 0 THEN 30
    RETURN  ' end of dah routine

REM     dit routine
35  OUT ctrlout, 6 'turn on the other paddle
dit = 0
40  stat = INP(statin) AND 128
    IF stat = 128 THEN 40
50  stat = INP(statin) AND 128
    dit = dit + 1
    IF stat = 0 THEN 50
    RETURN  ' end of dit routine

55 CLOSE 1
END

60
REM This subroutine defines the values for a morse code lookup table
REM contained in array a.  The idea for this table was taken from
REM the book: Z-80 Microcomputer Design Projects by William Barden.
REM
REM first element of array:  dah = 1, dit = 0, left justified
REM second element of array:  number of elements in the character
REM
REM These two elements are defined for each ASCII character of interest.

a(48, 1) = &HF8 '0 = 1111 1000 or f8
a(49, 1) = &H78 '1
a(50, 1) = &H38 '2
a(51, 1) = &H18 '3
a(52, 1) = &H8  '4
a(53, 1) = &H0  '5
a(54, 1) = &H80 '6
a(55, 1) = &HC0 '7
a(56, 1) = &HE0 '8
a(57, 1) = &HF0 '9

a(65, 1) = &H40 'A - upper case
a(66, 1) = &H80 'B
a(67, 1) = &HA0 'C
a(68, 1) = &H80 'D
a(69, 1) = &H0  'E
a(70, 1) = &H20 'F
a(71, 1) = &HC0 'G
a(72, 1) = &H0  'H
a(73, 1) = &H0  'I
a(74, 1) = &H70 'J
a(75, 1) = &HA0 'K
a(76, 1) = &H40 'L
a(77, 1) = &HC0 'M
a(78, 1) = &H80 'N
a(79, 1) = &HE0 'O
a(80, 1) = &H60 'P
a(81, 1) = &HD0 'Q
a(82, 1) = &H40 'R
a(83, 1) = &H0  'S
a(84, 1) = &H80 'T
a(85, 1) = &H20 'U
a(86, 1) = &H10 'V
a(87, 1) = &H60 'W
a(88, 1) = &H90 'X
a(89, 1) = &HB0 'Y
a(90, 1) = &HC0 'Z
         
a(97, 1) = &H40 'a - lower case
a(98, 1) = &H80 'b
a(99, 1) = &HA0 'c
a(100, 1) = &H80 'd
a(101, 1) = &H0  'e
a(102, 1) = &H20 'f
a(103, 1) = &HC0 'g
a(104, 1) = &H0  'h
a(105, 1) = &H0  'i
a(106, 1) = &H70 'j
a(107, 1) = &HA0 'k
a(108, 1) = &H40 'l
a(109, 1) = &HC0 'm
a(110, 1) = &H80 'n
a(111, 1) = &HE0 'o
a(112, 1) = &H60 'p
a(113, 1) = &HD0 'q
a(114, 1) = &H40 'r
a(115, 1) = &H0  's
a(116, 1) = &H80 't
a(117, 1) = &H20 'u
a(118, 1) = &H10 'v
a(119, 1) = &H60 'w
a(120, 1) = &H90 'x
a(121, 1) = &HB0 'y
a(122, 1) = &HC0 'z
REM               special characters
a(63, 1) = &H30 '?
a(44, 1) = &HCC ',
a(45, 1) = &H88 '- (bt)
a(46, 1) = &H54 '.
a(47, 1) = &H90 '/ (-..-.)
a(61, 1) = &H50 '= (ar)
a(59, 1) = &H14 '; (sk)
a(92, 1) = &H40 '\ (as)
a(96, 1) = &H0 '' (error)
a(58, 1) = &HFF ':  (8 dahs for embedding a pause into the Island Memory)

a(48, 2) = 5    '0 has 5 elements
a(49, 2) = 5    '1     5
a(50, 2) = 5    '2     5
a(51, 2) = 5    '3     5
a(52, 2) = 5    '4     5
a(53, 2) = 5    '5     5
a(54, 2) = 5    '6     5
a(55, 2) = 5    '7     5
a(56, 2) = 5    '8     5
a(57, 2) = 5    '9     5

 
a(97, 2) = 2  'a has 2 elements
a(98, 2) = 4  'b     4
a(99, 2) = 4  'c     4
a(100, 2) = 3 'd     3
a(101, 2) = 1 'e     1
a(102, 2) = 4 'f     4
a(103, 2) = 3 'g     3
a(104, 2) = 4 'h     4
a(105, 2) = 2 'i     2
a(106, 2) = 4 'j     4
a(107, 2) = 3 'k     3
a(108, 2) = 4 'l     4
a(109, 2) = 2 'm     2
a(110, 2) = 2 'n     2
a(111, 2) = 3 'o     3
a(112, 2) = 4 'p     4
a(113, 2) = 4 'q     4
a(114, 2) = 3 'r     3
a(115, 2) = 3 's     3
a(116, 2) = 1 't     1
a(117, 2) = 3 'u     3
a(118, 2) = 4 'v     4
a(119, 2) = 3 'w     3
a(120, 2) = 4 'x     4
a(121, 2) = 4 'y     4
a(122, 2) = 4 'z     4

a(65, 2) = 2 'upper case A has 2 elements
a(66, 2) = 4 'B     4
a(67, 2) = 4 'C     4
a(68, 2) = 3 'D     3
a(69, 2) = 1 'E     1
a(70, 2) = 4 'F     4
a(71, 2) = 3 'G     3
a(72, 2) = 4 'H     4
a(73, 2) = 2 'I     2
a(74, 2) = 4 'J     4
a(75, 2) = 3 'K     3
a(76, 2) = 4 'L     4
a(77, 2) = 2 'M     2
a(78, 2) = 2 'N     2
a(79, 2) = 3 'O     3
a(80, 2) = 4 'P     4
a(81, 2) = 4 'Q     4
a(82, 2) = 3 'R     3
a(83, 2) = 3 'S     3
a(84, 2) = 1 'T     1
a(85, 2) = 3 'U     3
a(86, 2) = 4 'V     4
a(87, 2) = 3 'W     3
a(88, 2) = 4 'X     4
a(89, 2) = 4 'Y     4
a(90, 2) = 4 'Z     4

a(63, 2) = 6 '?         6
a(44, 2) = 6 ',         6
a(45, 2) = 5 '- (bt)    5
a(46, 2) = 6 '.         6
a(47, 2) = 5 '/         5
a(61, 2) = 5 '= (ar)    5
a(59, 2) = 6 '; (sk)    6
a(92, 2) = 5 '\ (as)    5
a(96, 2) = 8 '' (error) 8
a(58, 2) = 8 ':        (8 dahs for embedding a pause into the Island Memory)
RETURN

